@@ -15,11 +15,12 @@
 //!   * For parsing use `str::parse::<Invoice>(&self)` (see the docs of `impl FromStr for Invoice`)
 //!   * For constructing invoices use the `InvoiceBuilder`
 //!   * For serializing invoices use the `Display`/`ToString` traits
+pub mod payment;
 pub mod utils;
 
 extern crate bech32;
 extern crate bitcoin_hashes;
-extern crate lightning;
+#[macro_use] extern crate lightning;
 extern crate num_traits;
 extern crate secp256k1;
 
@@ -378,7 +379,8 @@ pub enum TaggedField {
 
 /// SHA-256 hash
 #[derive(Clone, Debug, Hash, Eq, PartialEq)]
-pub struct Sha256(pub sha256::Hash);
+pub struct Sha256(/// (C-not exported) as the native hash types are not currently mapped
+	pub sha256::Hash);
 
 /// Description string
 ///
@@ -1187,6 +1189,19 @@ impl Invoice {
 			.unwrap_or(Duration::from_secs(DEFAULT_EXPIRY_TIME))
 	}
 
+	/// Returns whether the invoice has expired.
+	pub fn is_expired(&self) -> bool {
+		Self::is_expired_from_epoch(self.timestamp(), self.expiry_time())
+	}
+
+	/// Returns whether the expiry time from the given epoch has passed.
+	pub(crate) fn is_expired_from_epoch(epoch: &SystemTime, expiry_time: Duration) -> bool {
+		match epoch.elapsed() {
+			Ok(elapsed) => elapsed > expiry_time,
+			Err(_) => false,
+		}
+	}
+
 	/// Returns the invoice's `min_final_cltv_expiry` time, if present, otherwise
 	/// [`DEFAULT_MIN_FINAL_CLTV_EXPIRY`].
 	pub fn min_final_cltv_expiry(&self) -> u64 {
@@ -1208,10 +1223,10 @@ impl Invoice {
 	}
 
 	/// Returns a list of all routes included in the invoice as the underlying hints
-	pub fn route_hints(&self) -> Vec<&RouteHint> {
+	pub fn route_hints(&self) -> Vec<RouteHint> {
 		find_all_extract!(
 			self.signed_invoice.known_tagged_fields(), TaggedField::PrivateRoute(ref x), x
-		).map(|route| &**route).collect()
+		).map(|route| (**route).clone()).collect()
 	}
 
 	/// Returns the currency for which the invoice was issued
@@ -1219,8 +1234,13 @@ impl Invoice {
 		self.signed_invoice.currency()
 	}
 
+	/// Returns the amount if specified in the invoice as millisatoshis.
+	pub fn amount_milli_satoshis(&self) -> Option<u64> {
+		self.signed_invoice.amount_pico_btc().map(|v| v / 10)
+	}
+
 	/// Returns the amount if specified in the invoice as pico <currency>.
-	pub fn amount_pico_btc(&self) -> Option<u64> {
+	fn amount_pico_btc(&self) -> Option<u64> {
 		self.signed_invoice.amount_pico_btc()
 	}
 }
@@ -1867,6 +1887,7 @@ mod test {
 		assert!(invoice.check_signature().is_ok());
 		assert_eq!(invoice.tagged_fields().count(), 10);
 
+		assert_eq!(invoice.amount_milli_satoshis(), Some(123));
 		assert_eq!(invoice.amount_pico_btc(), Some(1230));
 		assert_eq!(invoice.currency(), Currency::BitcoinTestnet);
 		assert_eq!(
@@ -1913,5 +1934,33 @@ mod test {
 
 		assert_eq!(invoice.min_final_cltv_expiry(), DEFAULT_MIN_FINAL_CLTV_EXPIRY);
 		assert_eq!(invoice.expiry_time(), Duration::from_secs(DEFAULT_EXPIRY_TIME));
+		assert!(!invoice.is_expired());
+	}
+
+	#[test]
+	fn test_expiration() {
+		use ::*;
+		use secp256k1::Secp256k1;
+		use secp256k1::key::SecretKey;
+
+		let timestamp = SystemTime::now()
+			.checked_sub(Duration::from_secs(DEFAULT_EXPIRY_TIME * 2))
+			.unwrap();
+		let signed_invoice = InvoiceBuilder::new(Currency::Bitcoin)
+			.description("Test".into())
+			.payment_hash(sha256::Hash::from_slice(&[0;32][..]).unwrap())
+			.payment_secret(PaymentSecret([0; 32]))
+			.timestamp(timestamp)
+			.build_raw()
+			.unwrap()
+			.sign::<_, ()>(|hash| {
+				let privkey = SecretKey::from_slice(&[41; 32]).unwrap();
+				let secp_ctx = Secp256k1::new();
+				Ok(secp_ctx.sign_recoverable(hash, &privkey))
+			})
+			.unwrap();
+		let invoice = Invoice::from_signed(signed_invoice).unwrap();
+
+		assert!(invoice.is_expired());
 	}
 }
